function titr = weakweak5(CA,CB,volA,volB,pKAA,pKBB,hhplot)
% Plots a titration curve for titrating a weak acid with a weak base
% Outputs a matrix with columns volume of B,pH,[HA],[A-],[B],[BH+], buffer capacity
% Jason Kahn U Maryland 4/6/2017  version 1.02 (derived from weakweak4)
% [titr = ]weakweak5(CA,CB,volA,volB,pKAA,pKBB,hhplot);  for example:
% titr = weakweak5(.1,.05,100,225,4.75,3.5,0); % or just
% weakweak5(.75,1.0,300,250,6.15,-5,0); % for titration with a strong base
% if hhplot is set to 1, will also plot the pH derived from the
% Henderson-Hasselbalch just assuming complete neutralization to give [HA]
% and [A-].
% Titration of a weak acid A with concentration CA and volume volA with a
% weak base B of concentration (CB), from a volume of zero up to volB.
% KAA = 10^(-pKAA) is the acid dissociation constant of A and KBB = 10^(-pKBB)
% is the base dissociation constant of B. Volume units do not matter (as
% long as they are the same!), typically they would be ml.
% Returns a matrix with columns volB, pH,[HA],[A-],[B],[BH+],buffer capacity
% in the "titr" variable.
% The program graphs the first two versus each other to show a typical titration.
% Save the values returned in titr if you need to do multiple plots with different x
% scales. Otherwise the default is to overlay plots, so to do separate
% plots with different axes, give the "figure" command between calls to
% weakweak5.
% For example, to overlay curves showing the effect of dilution
% titr1 = weakweak5(.1,.03,100,400,4.75,3.5,1);
% titr2 = weakweak5(.01,.003,100,400,4.75,3.5,1);
% titr3 = weakweak5(.001,.0003,100,400,4.75,3.5,1);
% titr4 = weakweak5(.0001,.00003,100,400,4.75,3.5,1);
% To plot the distribution of HA, A-, B, BH+ species use these commands:
% >> titr2 = weakweak5(.1,.05,100,225,4.75,3.5,1);
% >> figure  % So you don't plot on top of the pH plot
% >> plot(titr2(:,1),titr2(:,3),titr2(:,1),titr2(:,4),titr2(:,1),titr2(:,5),titr2(:,1),titr2(:,6))
% >> legend('HA','A-','B','BH+')
%
KW = 1e-14;
KAA = 10^(-pKAA);
KBB = 10^(-pKBB);
KWoKB = KW/KBB;
%hhplot = 0; Should install this as a varargin etc.
npts = 301;  % This is a minimum number, probably must be > 200
% Figure out where the equivalence point is and make points denser in its vicinity. 
eqpt = volA*CA/CB;
disp(['Equiv pt at ',num2str(eqpt)])
if volB > 1.03*eqpt   
    Bvol = [ 0 : (volB-.06*eqpt)/(npts-101) : 0.97*eqpt-.06*eqpt/100 ,...
        0.97*eqpt : .06*eqpt/100 : eqpt-.06*eqpt/100 ,...
        eqpt-.06*eqpt/100+eqpt*2e-5 : eqpt*2e-5 : eqpt+.06*eqpt/100-eqpt*2e-5,...
        eqpt+.06*eqpt/100:.06*eqpt/100:1.03*eqpt ,...
        1.03*eqpt+.06*eqpt/100 : (volB-.06*eqpt)/(npts-101) : volB ];
    if Bvol(end) ~= volB
        Bvol = [Bvol volB];  % For the axes to plot neatly want the fixed end point
    end
    npts = length(Bvol);
    disp(['New npts ', num2str(npts)])
else
    Bvol = 0 : volB/(npts-1) : volB;
end
Bconcs = CB*Bvol./(volA+Bvol);
Aconcs = CA*volA./(volA+Bvol);
if (hhplot == 1)  % Plot the result of just plugging in to HH equation
    Hhmax = find(Bvol < eqpt,1,'last');
    Hhph = pKAA+log10(Bconcs(1:Hhmax)./(Aconcs(1:Hhmax)-Bconcs(1:Hhmax)));
end
proton = zeros(npts,1);
HA = zeros(npts,1);
Aminus = zeros(npts,1);
BaseB = zeros(npts,1);
BHplus =  zeros(npts,1);
pH = zeros(npts,1);
Bufcap = zeros(npts,1);
for i=1:npts
    a1 = 1;
    a2 = Bconcs(i) + KAA + KWoKB;
    a3 = KAA*(Bconcs(i) - Aconcs(i)) + KAA*KWoKB - KW;
    a4 = -Aconcs(i)*KAA*KWoKB - KW*(KAA + KWoKB);
    a5 = -KW*KAA*KWoKB;
    poly = [a1 a2 a3 a4 a5];
    rpoly = roots(poly);   % This is where all the action happens
    proton(i) = rpoly(abs(rpoly > 0) & imag(rpoly) == 0); % Choose the positive real root
    pH(i) = -log10(proton(i));
    % The HA calculation may be wrong in earlier versions?
    HA(i) = Aconcs(i)./(1+proton(i)/KAA);
    Aminus(i) = Aconcs(i)-HA(i);
    BaseB(i) = Bconcs(i)*(KW/(KBB*proton(i)))/(1+KW/(KBB*proton(i)));
    BHplus(i) = Bconcs(i)-BaseB(i);
    if i > 2
        Bufcap(i-1) = 0.5*((Bconcs(i)-Bconcs(i-1))/(pH(i)-pH(i-1)) ...
            + ((Bconcs(i-1)-Bconcs(i-2))/(pH(i-1)-pH(i-2))));
    end
end
% The buffer capacity calculation is dubious for a weak base titrant.
Bufcap(1) = (Bconcs(2)-Bconcs(1))/(pH(2)-pH(1));
Bufcap(npts) = (Bconcs(npts)-Bconcs(npts-1))/(pH(npts)-pH(npts-1));
%
titr = [Bvol' pH HA Aminus BaseB BHplus Bufcap];  % Returns a matrix with columns as shown
% The rest is plotting the pH vs. the added volume
%figure  % Uncomment this line to prevent overlaying plots, which is now
%the default. Should have an optional argument to control this from the
%command line.
hold on
plot(titr(:,1),titr(:,2),'LineWidth',2);
plot(titr(:,1),titr(:,2),'o');   % to see points
if (hhplot ==1)
    plot(Bvol(1:Hhmax),Hhph,'b-')
end
axis([0 max(Bvol) 0 14])    % So different titrations can be compared
xlimit = xlim;
numxticks = 8; % should make this adaptive somehow.
set(gca,'FontSize',14,'XTick',xlimit(1) : (xlimit(2)- xlimit(1))/numxticks...
    : xlimit(2))
title('Acid-Base Titration','FontWeight','bold','FontSize',16)
xlabel('Added volume of Base','FontWeight','bold','FontSize',14);
ylabel('pH','FontWeight','bold','FontSize',14);
text(0.10*max(Bvol),0.75,...
     ['pKa(A) = ',num2str(-log10(KAA),'%4.1f'),...
     '  pKb(B) = ',num2str(-log10(KBB),'%4.1f'), ...
     '  CA = ',num2str(CA),' M ',...
     '  CB = ',num2str(CB),' M ',...
     '  Init. vol of A = ',num2str(volA)],...
     'Fontsize',14,'FontWeight','bold')
% uncomment line below to save (should rewrite to use export_fig).
%print -dpng -r200
return
